@@ -48,6 +48,13 @@ impl LineGroup {
         let mut res = vec![];
         match &self.lhs_lines {
             Some(lhs_lines) => {
+                // TODO: it's not possible to represent an empty range
+                // with RangeInclusive and LineNumber. Define a
+                // separate interval type.
+                if lhs_lines.end().number == 0 {
+                    return vec![];
+                }
+
                 for line in lhs_lines.start().number..=lhs_lines.end().number {
                     res.push(line.into());
                 }
@@ -61,6 +68,13 @@ impl LineGroup {
         let mut res = vec![];
         match &self.rhs_lines {
             Some(rhs_lines) => {
+                // TODO: it's not possible to represent an empty range
+                // with RangeInclusive and LineNumber. Define a
+                // separate interval type.
+                if rhs_lines.end().number == 0 {
+                    return vec![];
+                }
+
                 for line in rhs_lines.start().number..=rhs_lines.end().number {
                     res.push(line.into());
                 }
@@ -460,15 +474,20 @@ pub fn apply_groups(
     groups: &[LineGroup],
     lhs_line_matches: &HashMap<LineNumber, LineNumber>,
     lhs_content_width: usize,
+    rhs_content_width: usize,
     lhs_column_width: usize,
     rhs_column_width: usize,
-    terminal_width: usize,
 ) -> String {
     let lhs_lines: Vec<_> = lhs.lines().collect();
     let rhs_lines: Vec<_> = rhs.lines().collect();
 
     let mut result = String::new();
-    let spacer = "-".repeat(terminal_width);
+
+    let mut spacer = String::new();
+    spacer.push_str(&" ".repeat(lhs_column_width));
+    spacer.push_str(&"-".repeat(lhs_content_width));
+    spacer.push_str(&" ".repeat(rhs_column_width + 2));
+    spacer.push_str(&"-".repeat(rhs_content_width));
 
     for (i, group) in groups.iter().enumerate() {
         result.push_str(&apply_group(
@@ -554,6 +573,32 @@ impl NewlinePositions {
         }
         res
     }
+
+    pub fn from_offsets_relative_to(
+        &self,
+        start: SingleLineSpan,
+        region_start: usize,
+        region_end: usize,
+    ) -> Vec<SingleLineSpan> {
+        let mut res = vec![];
+        for pos in self.from_offsets(region_start, region_end) {
+            if pos.line.number == 0 {
+                res.push(SingleLineSpan {
+                    line: start.line,
+                    start_col: start.start_col + pos.start_col,
+                    end_col: start.start_col + pos.end_col,
+                });
+            } else {
+                res.push(SingleLineSpan {
+                    line: (start.line.number + pos.line.number).into(),
+                    start_col: pos.start_col,
+                    end_col: pos.end_col,
+                });
+            }
+        }
+
+        res
+    }
 }
 
 #[test]
@@ -592,6 +637,27 @@ fn from_ranges_split_over_multiple_lines() {
     );
 }
 
+#[test]
+fn from_offsets_relative_to() {
+    let newline_positions: NewlinePositions = "foo\nbar".into();
+
+    let pos = SingleLineSpan {
+        line: 1.into(),
+        start_col: 1,
+        end_col: 1,
+    };
+
+    let line_spans = newline_positions.from_offsets_relative_to(pos, 1, 2);
+    assert_eq!(
+        line_spans,
+        vec![SingleLineSpan {
+            line: 1.into(),
+            start_col: 2,
+            end_col: 3
+        }]
+    );
+}
+
 /// Ensure that every line in `s` has this length. Pad short lines and
 /// truncate long lines.
 pub fn enforce_length(s: &str, line_length: usize) -> String {
@@ -629,7 +695,7 @@ pub trait MaxLine {
 
 impl MaxLine for String {
     fn max_line(&self) -> LineNumber {
-        (self.lines().count() - 1).into()
+        (max(1, self.lines().count()) - 1).into()
     }
 }
 
@@ -638,3 +704,9 @@ fn str_max_line() {
     let line: String = "foo\nbar".into();
     assert_eq!(line.max_line().number, 1);
 }
+
+#[test]
+fn empty_str_max_line() {
+    let line: String = "".into();
+    assert_eq!(line.max_line().number, 0);
+}
