@@ -16,16 +16,18 @@ use crate::lines::{
 };
 use crate::parse::{find_lang, parse, parse_lines, read_or_die, ConfigDir};
 use crate::style::apply_colors;
-use crate::syntax::{change_positions, matching_lines, set_next};
+use crate::syntax::{change_positions, init_info, matching_lines};
 
 fn term_width() -> Option<usize> {
     term_size::dimensions().map(|(w, _)| w)
 }
 
+const VERSION: &'static str = env!("CARGO_PKG_VERSION");
+
 fn main() {
     let matches = App::new("Difftastic")
-        .version("0.1")
-        .about("A word level diff tool that understands syntax!")
+        .version(VERSION)
+        .about("A syntax aware diff.")
         .author("Wilfred Hughes")
         .arg(
             Arg::with_name("LANGUAGE")
@@ -50,24 +52,30 @@ fn main() {
                 .long("inline")
                 .help("Prefer single column output"),
         )
-        .arg(Arg::with_name("first").index(1).required(true))
-        .arg(Arg::with_name("second").index(2).required(true))
+        .arg(Arg::with_name("positional_args").multiple(true))
         .get_matches();
 
-    let lhs_path = matches.value_of("first").unwrap();
-    let lhs_src = read_or_die(lhs_path);
+    let args: Vec<_> = matches.values_of_lossy("positional_args").unwrap();
 
-    let rhs_path = matches.value_of("second").unwrap();
-    let rhs_src = read_or_die(rhs_path);
+    // TODO: document these different ways of calling difftastic.
+    let (display_path, lhs_path, rhs_path) = match &args[..] {
+        [lhs_path, rhs_path] => (rhs_path, lhs_path, rhs_path),
+        [display_path, lhs_tmp_file, _lhs_hash, _lhs_mode, rhs_tmp_file, _rhs_hash, _rhs_mode] => {
+            // https://git-scm.com/docs/git#Documentation/git.txt-codeGITEXTERNALDIFFcode
+            (display_path, lhs_tmp_file, rhs_tmp_file)
+        }
+        _ => panic!("Expected 2 arguments or 7 arguments"),
+    };
+
+    let lhs_src = read_or_die(&lhs_path);
+    let rhs_src = read_or_die(&rhs_path);
 
     let syntax_toml = ConfigDir::read_default_toml();
 
-    let lhs_extension = Path::new(lhs_path)
-        .extension()
-        .and_then(OsStr::to_str)
-        .unwrap();
-
-    let lang = find_lang(syntax_toml, lhs_extension);
+    let lang = match Path::new(&display_path).extension() {
+        Some(extension) => find_lang(syntax_toml, &OsStr::to_string_lossy(extension)),
+        None => None,
+    };
 
     let terminal_width = match matches.value_of("COLUMNS") {
         Some(width) => width.parse::<usize>().unwrap(),
@@ -84,9 +92,9 @@ fn main() {
         None => (parse_lines(&arena, &lhs_src), parse_lines(&arena, &rhs_src)),
     };
 
-    set_next(&lhs, None);
-    set_next(&rhs, None);
-    mark_syntax(lhs.get(0).map(|n| *n), rhs.get(0).map(|n| *n));
+    init_info(&lhs);
+    init_info(&rhs);
+    mark_syntax(lhs.get(0).copied(), rhs.get(0).copied());
 
     let lhs_positions = change_positions(&lhs_src, &rhs_src, &lhs);
     let rhs_positions = change_positions(&rhs_src, &lhs_src, &rhs);
@@ -97,11 +105,15 @@ fn main() {
         Some(lang) => lang.name.clone(),
         None => "plain text".to_string(),
     };
-    println!("{}", style::header(rhs_path, &lang_name));
+    println!("{}", style::header(&display_path, &lang_name));
 
     let mut groups = visible_groups(&lhs_positions, &rhs_positions);
     if groups.is_empty() {
-        println!("No changes found.");
+        if lang.is_some() {
+            println!("No syntactic changes.");
+        } else {
+            println!("No changes.");
+        }
         return;
     }
 
@@ -137,9 +149,9 @@ fn main() {
             &groups,
             &lhs_matched_lines,
             lhs_content_width,
+            rhs_content_width,
             lhs_column_width,
             rhs_column_width,
-            terminal_width
         )
     );
     println!();
