@@ -1,6 +1,6 @@
 use crate::lines::NewlinePositions;
 use crate::positions::SingleLineSpan;
-use crate::syntax::{AtomKind, Syntax};
+use crate::syntax::Syntax;
 use regex::Regex;
 use rust_embed::RustEmbed;
 use std::fs;
@@ -8,8 +8,8 @@ use toml::Value;
 use typed_arena::Arena;
 
 pub fn read_or_die(path: &str) -> String {
-    match fs::read_to_string(path) {
-        Ok(src) => src,
+    match fs::read(path) {
+        Ok(src) => String::from_utf8_lossy(&src).to_string(),
         Err(e) => {
             match e.kind() {
                 std::io::ErrorKind::NotFound => {
@@ -80,11 +80,7 @@ fn as_regex_vec(v: &Value) -> Vec<Regex> {
 }
 
 fn as_regex(s: &str) -> Regex {
-    let mut pattern = String::with_capacity(1 + s.len());
-    pattern.push('^');
-    pattern.push_str(s);
-
-    Regex::new(&pattern).unwrap()
+    Regex::new(&s).unwrap()
 }
 
 fn lang_from_value(name: &str, v: &Value) -> Language {
@@ -116,44 +112,17 @@ fn lang_from_value(name: &str, v: &Value) -> Language {
 ///
 /// This is a fallback for files that we don't know how to parse.
 pub fn parse_lines<'a>(arena: &'a Arena<Syntax<'a>>, s: &str) -> Vec<&'a Syntax<'a>> {
-    let mut line_start = 0;
     let mut res: Vec<&'a Syntax<'a>> = vec![];
-
-    for (i, c) in s.chars().enumerate() {
-        if c == '\n' {
-            let line = &s[line_start..i];
-            let atom = Syntax::new_atom(
-                arena,
-                vec![SingleLineSpan {
-                    line: res.len().into(),
-                    start_col: 0,
-                    end_col: i - line_start,
-                }],
-                line,
-                AtomKind::Other,
-            );
-            res.push(atom);
-
-            line_start = i + 1;
-        }
-    }
-
-    if let Some(last) = s.chars().last() {
-        if last != '\n' {
-            let line = &s[line_start..];
-
-            let atom = Syntax::new_atom(
-                arena,
-                vec![SingleLineSpan {
-                    line: res.len().into(),
-                    start_col: 0,
-                    end_col: s.len() - line_start,
-                }],
-                line,
-                AtomKind::Other,
-            );
-            res.push(atom);
-        }
+    for (i, line) in s.lines().enumerate() {
+        res.push(Syntax::new_atom(
+            arena,
+            vec![SingleLineSpan {
+                line: i.into(),
+                start_col: 0,
+                end_col: line.len(),
+            }],
+            line,
+        ));
     }
 
     res
@@ -165,6 +134,13 @@ pub fn parse<'a>(arena: &'a Arena<Syntax<'a>>, s: &str, lang: &Language) -> Vec<
     parse_from(arena, s, &nl_pos, lang, &mut ParseState::new())
 }
 
+enum LexKind {
+    Comment,
+    Atom,
+    OpenDelimiter,
+    CloseDelimiter,
+}
+
 fn parse_from<'a>(
     arena: &'a Arena<Syntax<'a>>,
     s: &str,
@@ -174,84 +150,113 @@ fn parse_from<'a>(
 ) -> Vec<&'a Syntax<'a>> {
     let mut result: Vec<&'a Syntax<'a>> = vec![];
 
-    'outer: while state.str_i < s.len() {
+    while state.str_i < s.len() {
+        let mut current_match: Option<(LexKind, regex::Match)> = None;
+
         for pattern in &lang.comment_patterns {
             if let Some(m) = pattern.find(&s[state.str_i..]) {
-                assert_eq!(m.start(), 0);
-                let atom = Syntax::new_atom(
-                    arena,
-                    nl_pos.from_offsets(state.str_i, state.str_i + m.end()),
-                    m.as_str(),
-                    AtomKind::Comment,
-                );
-                result.push(atom);
-                state.str_i += m.end();
-                continue 'outer;
+                match current_match {
+                    Some((_, prev_m)) if prev_m.start() <= m.start() => {}
+                    _ => {
+                        current_match = Some((LexKind::Comment, m));
+                    }
+                }
             }
         }
 
         for pattern in &lang.atom_patterns {
             if let Some(m) = pattern.find(&s[state.str_i..]) {
-                assert_eq!(m.start(), 0);
-                let atom = Syntax::new_atom(
-                    arena,
-                    nl_pos.from_offsets(state.str_i, state.str_i + m.end()),
-                    m.as_str(),
-                    AtomKind::Other,
-                );
-                result.push(atom);
-                state.str_i += m.end();
-                continue 'outer;
+                match current_match {
+                    Some((_, prev_m)) if prev_m.start() <= m.start() => {}
+                    _ => {
+                        current_match = Some((LexKind::Atom, m));
+                    }
+                }
             }
         }
 
+        // TODO: fix duplication with previous loop
         for pattern in &lang.string_patterns {
             if let Some(m) = pattern.find(&s[state.str_i..]) {
-                assert_eq!(m.start(), 0);
-                let atom = Syntax::new_atom(
-                    arena,
-                    nl_pos.from_offsets(state.str_i, state.str_i + m.end()),
-                    m.as_str(),
-                    AtomKind::String,
-                );
-                result.push(atom);
-                state.str_i += m.end();
-                continue 'outer;
+                match current_match {
+                    Some((_, prev_m)) if prev_m.start() <= m.start() => {}
+                    _ => {
+                        current_match = Some((LexKind::Atom, m));
+                    }
+                }
             }
         }
 
         if let Some(m) = lang.open_delimiter_pattern.find(&s[state.str_i..]) {
-            let start = state.str_i;
-
-            state.str_i += m.end();
-            let children = parse_from(arena, s, nl_pos, lang, state);
-            let (close_brace, close_pos) = state.close_brace.take().unwrap_or((
-                "UNCLOSED".into(),
-                nl_pos.from_offsets(state.str_i, state.str_i + 1),
-            ));
-
-            let open_pos = nl_pos.from_offsets(start, start + m.end());
-            let items = Syntax::new_list(
-                arena,
-                m.as_str(),
-                open_pos,
-                children,
-                &close_brace,
-                close_pos,
-            );
-            result.push(items);
-            continue;
+            match current_match {
+                Some((_, prev_m)) if prev_m.start() <= m.start() => {}
+                _ => {
+                    current_match = Some((LexKind::OpenDelimiter, m));
+                }
+            }
         };
 
         if let Some(m) = lang.close_delimiter_pattern.find(&s[state.str_i..]) {
-            state.close_brace = Some((
-                m.as_str().into(),
-                nl_pos.from_offsets(state.str_i, state.str_i + m.end()),
-            ));
-            state.str_i += m.end();
-            return result;
+            match current_match {
+                Some((_, prev_m)) if prev_m.start() <= m.start() => {}
+                _ => {
+                    current_match = Some((LexKind::CloseDelimiter, m));
+                }
+            }
+        };
+
+        match current_match {
+            Some((match_kind, m)) => match match_kind {
+                LexKind::Comment => {
+                    let atom = Syntax::new_comment(
+                        arena,
+                        nl_pos.from_offsets(state.str_i + m.start(), state.str_i + m.end()),
+                        m.as_str(),
+                    );
+                    result.push(atom);
+                    state.str_i += m.end();
+                }
+                LexKind::Atom => {
+                    let atom = Syntax::new_atom(
+                        arena,
+                        nl_pos.from_offsets(state.str_i + m.start(), state.str_i + m.end()),
+                        m.as_str(),
+                    );
+                    result.push(atom);
+                    state.str_i += m.end();
+                }
+                LexKind::OpenDelimiter => {
+                    let start = state.str_i;
+
+                    state.str_i += m.end();
+                    let children = parse_from(arena, s, nl_pos, lang, state);
+                    let (close_brace, close_pos) = state.close_brace.take().unwrap_or((
+                        "UNCLOSED".into(),
+                        nl_pos.from_offsets(state.str_i, state.str_i + 1),
+                    ));
+
+                    let open_pos = nl_pos.from_offsets(start + m.start(), start + m.end());
+                    let items = Syntax::new_list(
+                        arena,
+                        m.as_str(),
+                        open_pos,
+                        children,
+                        &close_brace,
+                        close_pos,
+                    );
+                    result.push(items);
+                }
+                LexKind::CloseDelimiter => {
+                    state.close_brace = Some((
+                        m.as_str().into(),
+                        nl_pos.from_offsets(state.str_i + m.start(), state.str_i + m.end()),
+                    ));
+                    state.str_i += m.end();
+                    return result;
+                }
+            },
+            None => break,
         };
-        state.str_i += 1;
     }
 
     result
@@ -308,28 +313,28 @@ mod tests {
         match (actual, expected) {
             (
                 List {
-                    change: lhs_change,
+                    info: lhs_info,
                     open_position: lhs_open_position,
-                    open_delimiter: lhs_start_content,
+                    open_content: lhs_start_content,
                     children: lhs_children,
-                    close_delimiter: lhs_end_content,
+                    close_content: lhs_end_content,
                     close_position: lhs_close_position,
                     num_descendants: lhs_num_descendants,
                     ..
                 },
                 List {
-                    change: rhs_change,
+                    info: rhs_info,
                     open_position: rhs_open_position,
-                    open_delimiter: rhs_start_content,
+                    open_content: rhs_start_content,
                     children: rhs_children,
-                    close_delimiter: rhs_end_content,
+                    close_content: rhs_end_content,
                     close_position: rhs_close_position,
                     num_descendants: rhs_num_descendants,
                     ..
                 },
             ) => {
-                if lhs_change != rhs_change {
-                    dbg!(lhs_change.get(), rhs_change.get());
+                if lhs_info.change.get() != rhs_info.change.get() {
+                    dbg!(lhs_info.change.get(), rhs_info.change.get());
                     return false;
                 }
                 if lhs_open_position != rhs_open_position {
@@ -361,22 +366,22 @@ mod tests {
             }
             (
                 Atom {
-                    change: lhs_change,
+                    info: lhs_info,
                     position: lhs_position,
                     content: lhs_content,
-                    kind: lhs_kind,
+                    is_comment: lhs_is_comment,
                     ..
                 },
                 Atom {
-                    change: rhs_change,
+                    info: rhs_info,
                     position: rhs_position,
                     content: rhs_content,
-                    kind: rhs_kind,
+                    is_comment: rhs_is_comment,
                     ..
                 },
             ) => {
-                if lhs_change != rhs_change {
-                    dbg!(lhs_change.get(), rhs_change.get());
+                if lhs_info.change.get() != rhs_info.change.get() {
+                    dbg!(lhs_info.change.get(), rhs_info.change.get());
                     return false;
                 }
                 if lhs_position != rhs_position {
@@ -388,8 +393,8 @@ mod tests {
                     dbg!(lhs_content, rhs_content);
                     return false;
                 }
-                if lhs_kind != rhs_kind {
-                    dbg!(lhs_kind, rhs_kind);
+                if lhs_is_comment != rhs_is_comment {
+                    dbg!(lhs_is_comment, rhs_is_comment);
                     return false;
                 }
             }
@@ -415,7 +420,6 @@ mod tests {
                         end_col: 3,
                     }],
                     "foo",
-                    AtomKind::Other,
                 ),
                 Syntax::new_atom(
                     &arena,
@@ -425,7 +429,6 @@ mod tests {
                         end_col: 3,
                     }],
                     "bar",
-                    AtomKind::Other,
                 ),
             ],
         );
@@ -445,7 +448,6 @@ mod tests {
                     end_col: 3,
                 }],
                 "123",
-                AtomKind::Other,
             )],
         );
     }
@@ -464,7 +466,93 @@ mod tests {
                     end_col: 2,
                 }],
                 "\"\"",
-                AtomKind::String,
+            )],
+        );
+    }
+
+    #[test]
+    fn test_parse_string_escaped_doublequote() {
+        let arena = Arena::new();
+
+        assert_syntaxes(
+            // "\""
+            &parse(&arena, "\"\\\"\"", &lang()),
+            &[Syntax::new_atom(
+                &arena,
+                vec![SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 4,
+                }],
+                "\"\\\"\"",
+            )],
+        );
+    }
+
+    #[test]
+    fn test_parse_string_escaped_backlash() {
+        let arena = Arena::new();
+
+        assert_syntaxes(
+            // "\\"
+            &parse(&arena, "\"\\\\\"", &lang()),
+            &[Syntax::new_atom(
+                &arena,
+                vec![SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 4,
+                }],
+                "\"\\\\\"",
+            )],
+        );
+    }
+
+    #[test]
+    fn test_parse_multiline_string() {
+        let arena = Arena::new();
+
+        assert_syntaxes(
+            &parse(&arena, "\"\n\"", &lang()),
+            &[Syntax::new_atom(
+                &arena,
+                vec![SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 1,
+                }, SingleLineSpan {
+                    line: 1.into(),
+                    start_col: 0,
+                    end_col: 1,
+                }],
+                "\"\n\"",
+            )],
+        );
+    }
+
+    #[test]
+    fn test_parse_string_escaped_backlash_and_second_string() {
+        let arena = Arena::new();
+
+        assert_syntaxes(
+            // "\\" "a"
+            &parse(&arena, "\"\\\\\" \"a\"", &lang()),
+            &[Syntax::new_atom(
+                &arena,
+                vec![SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 4,
+                }],
+                "\"\\\\\"",
+            ), Syntax::new_atom(
+                &arena,
+                vec![SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 5,
+                    end_col: 8,
+                }],
+                "\"a\"",
             )],
         );
     }
@@ -484,7 +572,6 @@ mod tests {
                         end_col: 3,
                     }],
                     "123",
-                    AtomKind::Other,
                 ),
                 Syntax::new_atom(
                     &arena,
@@ -494,7 +581,6 @@ mod tests {
                         end_col: 7,
                     }],
                     "456",
-                    AtomKind::Other,
                 ),
             ],
         );
@@ -514,7 +600,6 @@ mod tests {
                     end_col: 4,
                 }],
                 ".foo",
-                AtomKind::Other,
             )],
         );
     }
@@ -533,7 +618,6 @@ mod tests {
                     end_col: 4,
                 }],
                 "123",
-                AtomKind::Other,
             )],
         );
     }
@@ -552,7 +636,6 @@ mod tests {
                     end_col: 5,
                 }],
                 "\"abc\"",
-                AtomKind::String,
             )],
         );
     }
@@ -564,7 +647,7 @@ mod tests {
         assert_syntaxes(
             &parse(&arena, "// foo\nx", &lang()),
             &[
-                Syntax::new_atom(
+                Syntax::new_comment(
                     &arena,
                     vec![SingleLineSpan {
                         line: 0.into(),
@@ -572,7 +655,6 @@ mod tests {
                         end_col: 6,
                     }],
                     "// foo\n",
-                    AtomKind::Comment,
                 ),
                 Syntax::new_atom(
                     &arena,
@@ -582,7 +664,6 @@ mod tests {
                         end_col: 1,
                     }],
                     "x",
-                    AtomKind::Other,
                 ),
             ],
         );
@@ -594,7 +675,7 @@ mod tests {
 
         assert_syntaxes(
             &parse(&arena, "/* foo\nbar */", &lang()),
-            &[Syntax::new_atom(
+            &[Syntax::new_comment(
                 &arena,
                 vec![
                     SingleLineSpan {
@@ -609,7 +690,6 @@ mod tests {
                     },
                 ],
                 "/* foo\nbar */",
-                AtomKind::Comment,
             )],
         );
     }
@@ -636,7 +716,6 @@ mod tests {
                         end_col: 5,
                     }],
                     "123",
-                    AtomKind::Other,
                 )],
                 "]",
                 vec![SingleLineSpan {
@@ -720,7 +799,6 @@ mod tests {
                             end_col: 4,
                         }],
                         "123",
-                        AtomKind::Other,
                     ),
                     Syntax::new_atom(
                         &arena,
@@ -730,7 +808,6 @@ mod tests {
                             end_col: 5,
                         }],
                         ",",
-                        AtomKind::Other,
                     ),
                     Syntax::new_atom(
                         &arena,
@@ -740,7 +817,6 @@ mod tests {
                             end_col: 9,
                         }],
                         "456",
-                        AtomKind::Other,
                     ),
                 ],
                 "]",
@@ -776,7 +852,6 @@ mod tests {
                             end_col: 2,
                         }],
                         "x",
-                        AtomKind::Other,
                     ),
                     Syntax::new_atom(
                         &arena,
@@ -786,7 +861,6 @@ mod tests {
                             end_col: 3,
                         }],
                         ":",
-                        AtomKind::Other,
                     ),
                     Syntax::new_atom(
                         &arena,
@@ -796,7 +870,6 @@ mod tests {
                             end_col: 5,
                         }],
                         "1",
-                        AtomKind::Other,
                     ),
                 ],
                 "}",
