@@ -1,11 +1,11 @@
 #![allow(clippy::mutable_key_type)] // Hash for Syntax doesn't use mutable fields.
-#![allow(dead_code)]
 
 use itertools::{EitherOrBoth, Itertools};
+use regex::Regex;
 use std::cell::Cell;
 use std::cmp::min;
 use std::collections::hash_map::DefaultHasher;
-use std::collections::{HashMap, HashSet};
+use std::collections::HashMap;
 use std::fmt;
 use std::hash::{Hash, Hasher};
 use typed_arena::Arena;
@@ -18,7 +18,7 @@ use Syntax::*;
 #[derive(PartialEq, Eq, Clone, Copy)]
 pub enum ChangeKind<'a> {
     Unchanged(&'a Syntax<'a>),
-    Moved,
+    ReplacedComment(&'a Syntax<'a>, &'a Syntax<'a>),
     Novel,
 }
 
@@ -29,61 +29,77 @@ impl<'a> fmt::Debug for ChangeKind<'a> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let desc = match self {
             Unchanged(_) => "Unchanged",
-            Moved => "Moved",
+            ReplacedComment(_, _) => "ReplacedComment",
             Novel => "Novel",
         };
         f.write_str(desc)
     }
 }
 
-#[derive(PartialEq, Eq, Copy, Clone, Debug, PartialOrd, Ord)]
-pub enum AtomKind {
-    String,
-    Comment,
-    Other,
+/// Fields that are common to both `Syntax::List` and `Syntax::Atom`.
+pub struct SyntaxInfo<'a> {
+    pub pos_content_hash: u64,
+    pub next: Cell<Option<&'a Syntax<'a>>>,
+    pub change: Cell<Option<ChangeKind<'a>>>,
+    pub num_ancestors: Cell<u64>,
+    pub unique_id: Cell<u64>,
 }
 
 pub enum Syntax<'a> {
     List {
-        pos_content_hash: u64,
-        next: Cell<Option<&'a Syntax<'a>>>,
-        change: Cell<Option<ChangeKind<'a>>>,
+        info: SyntaxInfo<'a>,
         open_position: Vec<SingleLineSpan>,
         open_content: String,
         children: Vec<&'a Syntax<'a>>,
         close_position: Vec<SingleLineSpan>,
         close_content: String,
+        // TODO: this probably makes more sense as a u64, since we
+        // don't use it for indexing.
         num_descendants: usize,
     },
     Atom {
-        pos_content_hash: u64,
-        next: Cell<Option<&'a Syntax<'a>>>,
-        change: Cell<Option<ChangeKind<'a>>>,
+        info: SyntaxInfo<'a>,
         position: Vec<SingleLineSpan>,
         content: String,
-        kind: AtomKind,
+        is_comment: bool,
     },
 }
 
+fn dbg_pos(pos: &[SingleLineSpan]) -> String {
+    if pos.is_empty() {
+        "-".into()
+    } else {
+        let start = pos.first().unwrap();
+        let end = pos.last().unwrap();
+        format!(
+            "{}:{}-{}:{}",
+            start.line.number, start.start_col, end.line.number, end.end_col
+        )
+    }
+}
+
 impl<'a> fmt::Debug for Syntax<'a> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             List {
                 open_content,
+                open_position,
                 children,
                 close_content,
-                change,
-                next,
+                close_position,
+                info,
                 ..
             } => {
-                let mut ds = f.debug_struct("List");
+                let mut ds = f.debug_struct(&format!("List id:{}", self.id()));
 
                 ds.field("open_content", &open_content)
+                    .field("open_position", &dbg_pos(open_position))
                     .field("children", &children)
                     .field("close_content", &close_content)
-                    .field("change", &change.get());
+                    .field("close_position", &dbg_pos(close_position))
+                    .field("change", &info.change.get());
 
-                let next_s = match next.get() {
+                let next_s = match info.next.get() {
                     Some(List { .. }) => "Some(List)",
                     Some(Atom { .. }) => "Some(Atom)",
                     None => "None",
@@ -94,16 +110,16 @@ impl<'a> fmt::Debug for Syntax<'a> {
             }
             Atom {
                 content,
-                change,
-                next,
                 position,
+                info,
                 ..
             } => {
-                let mut ds = f.debug_struct("Atom");
-                ds.field("content", &content).field("change", &change.get());
-                ds.field("position", &position);
+                let mut ds = f.debug_struct(&format!("Atom id:{}", self.id()));
+                ds.field("content", &content)
+                    .field("change", &info.change.get());
+                ds.field("position", &dbg_pos(position));
 
-                let next_s = match next.get() {
+                let next_s = match info.next.get() {
                     Some(List { .. }) => "Some(List)",
                     Some(Atom { .. }) => "Some(Atom)",
                     None => "None",
@@ -147,9 +163,13 @@ impl<'a> Syntax<'a> {
         }
 
         arena.alloc(List {
-            pos_content_hash: hasher.finish(),
-            next: Cell::new(None),
-            change: Cell::new(None),
+            info: SyntaxInfo {
+                unique_id: Cell::new(0),
+                pos_content_hash: hasher.finish(),
+                next: Cell::new(None),
+                change: Cell::new(None),
+                num_ancestors: Cell::new(0),
+            },
             open_position,
             open_content: open_content.into(),
             close_content: close_content.into(),
@@ -164,7 +184,25 @@ impl<'a> Syntax<'a> {
         arena: &'a Arena<Syntax<'a>>,
         position: Vec<SingleLineSpan>,
         content: &str,
-        kind: AtomKind,
+    ) -> &'a mut Syntax<'a> {
+        Self::new_atom_(arena, position, content, false)
+    }
+
+    #[allow(clippy::mut_from_ref)] // Clippy doesn't understand arenas.
+    pub fn new_comment(
+        arena: &'a Arena<Syntax<'a>>,
+        position: Vec<SingleLineSpan>,
+        content: &str,
+    ) -> &'a mut Syntax<'a> {
+        Self::new_atom_(arena, position, content, true)
+    }
+
+    #[allow(clippy::mut_from_ref)] // Clippy doesn't understand arenas.
+    fn new_atom_(
+        arena: &'a Arena<Syntax<'a>>,
+        position: Vec<SingleLineSpan>,
+        content: &str,
+        is_comment: bool,
     ) -> &'a mut Syntax<'a> {
         let mut hasher = DefaultHasher::new();
 
@@ -172,48 +210,52 @@ impl<'a> Syntax<'a> {
         content.hash(&mut hasher);
 
         arena.alloc(Atom {
-            pos_content_hash: hasher.finish(),
-            next: Cell::new(None),
+            info: SyntaxInfo {
+                unique_id: Cell::new(0),
+                pos_content_hash: hasher.finish(),
+                next: Cell::new(None),
+                change: Cell::new(None),
+                num_ancestors: Cell::new(0),
+            },
             position,
             content: content.into(),
-            change: Cell::new(None),
-            kind,
+            is_comment,
         })
     }
 
-    pub fn get_next(&self) -> Option<&'a Syntax<'a>> {
+    pub fn info(&self) -> &SyntaxInfo<'a> {
         match self {
-            List { next, .. } => next.get(),
-            Atom { next, .. } => next.get(),
+            List { info, .. } => info,
+            Atom { info, .. } => info,
         }
     }
 
-    pub fn next_on_same_line(&self) -> bool {
-        match self.get_next() {
-            Some(next) => {
-                let self_last_line = match self {
-                    List { close_position, .. } => close_position.last().map(|lp| lp.line),
-                    Atom { position, .. } => position.last().map(|lp| lp.line),
-                };
-                let next_first_line = match next {
-                    List { open_position, .. } => open_position.first().map(|lp| lp.line),
-                    Atom { position, .. } => position.first().map(|lp| lp.line),
-                };
-                self_last_line == next_first_line
-            }
-            None => false,
-        }
+    pub fn next(&self) -> Option<&'a Syntax<'a>> {
+        self.info().next.get()
+    }
+
+    pub fn id(&self) -> u64 {
+        self.info().unique_id.get()
+    }
+
+    pub fn first_line(&self) -> Option<LineNumber> {
+        let position = match self {
+            List { open_position, .. } => open_position,
+            Atom { position, .. } => position,
+        };
+        position.first().map(|lp| lp.line)
+    }
+
+    pub fn last_line(&self) -> Option<LineNumber> {
+        let position = match self {
+            List { close_position, .. } => close_position,
+            Atom { position, .. } => position,
+        };
+        position.last().map(|lp| lp.line)
     }
 
     pub fn set_change(&self, ck: ChangeKind<'a>) {
-        match self {
-            List { change, .. } => {
-                change.set(Some(ck));
-            }
-            Atom { change, .. } => {
-                change.set(Some(ck));
-            }
-        }
+        self.info().change.set(Some(ck));
     }
 
     pub fn set_change_deep(&self, ck: ChangeKind<'a>) {
@@ -243,15 +285,15 @@ impl<'a> Syntax<'a> {
             (
                 Atom {
                     content: lhs_content,
-                    kind: lhs_kind,
+                    is_comment: lhs_is_comment,
                     ..
                 },
                 Atom {
                     content: rhs_content,
-                    kind: rhs_kind,
+                    is_comment: rhs_is_comment,
                     ..
                 },
-            ) => lhs_content == rhs_content && lhs_kind == rhs_kind,
+            ) => lhs_content == rhs_content && lhs_is_comment == rhs_is_comment,
             (
                 List {
                     open_content: lhs_open_content,
@@ -336,21 +378,44 @@ impl<'a> Syntax<'a> {
     }
 }
 
-pub fn set_next<'a>(nodes: &[&'a Syntax<'a>], parent_next: Option<&'a Syntax<'a>>) {
+pub fn init_info<'a>(roots: &[&'a Syntax<'a>]) {
+    set_unique_id(roots, 0);
+    set_next(roots, None);
+    set_num_ancestors(roots, 0);
+}
+
+fn set_unique_id<'a>(nodes: &[&'a Syntax<'a>], prev_id: u64) -> u64 {
+    let mut id = prev_id + 1;
+    for node in nodes {
+        node.info().unique_id.set(id);
+        if let List { children, .. } = node {
+            id = set_unique_id(children, id);
+        }
+        id += 1;
+    }
+    id
+}
+
+fn set_next<'a>(nodes: &[&'a Syntax<'a>], parent_next: Option<&'a Syntax<'a>>) {
     for (i, node) in nodes.iter().enumerate() {
         let node_next = match nodes.get(i + 1) {
             Some(node_next) => Some(*node_next),
             None => parent_next,
         };
 
-        match node {
-            List { next, children, .. } => {
-                next.set(node_next);
-                set_next(children, node_next);
-            }
-            Atom { next, .. } => {
-                next.set(node_next);
-            }
+        node.info().next.set(node_next);
+        if let List { children, .. } = node {
+            set_next(children, node_next);
+        }
+    }
+}
+
+fn set_num_ancestors<'a>(nodes: &[&Syntax<'a>], num_ancestors: u64) {
+    for node in nodes {
+        node.info().num_ancestors.set(num_ancestors);
+
+        if let List { children, .. } = node {
+            set_num_ancestors(children, num_ancestors + 1);
         }
     }
 }
@@ -364,36 +429,16 @@ impl<'a> Eq for Syntax<'a> {}
 
 impl<'a> Hash for Syntax<'a> {
     fn hash<H: Hasher>(&self, state: &mut H) {
-        match self {
-            List {
-                pos_content_hash, ..
-            } => {
-                pos_content_hash.hash(state);
-            }
-            Atom {
-                pos_content_hash, ..
-            } => {
-                pos_content_hash.hash(state);
-            }
-        }
+        self.info().pos_content_hash.hash(state);
     }
 }
 
 #[derive(PartialEq, Eq, Debug)]
 pub enum MatchKind {
     Unchanged,
-    Moved,
     Novel,
-}
-
-impl MatchKind {
-    fn from_change(ck: ChangeKind) -> Self {
-        match ck {
-            Unchanged(_) => MatchKind::Unchanged,
-            Moved => MatchKind::Moved,
-            Novel => MatchKind::Novel,
-        }
-    }
+    UnchangedCommentPart,
+    ChangedCommentPart,
 }
 
 #[derive(Debug)]
@@ -403,6 +448,102 @@ pub struct MatchedPos {
     pub prev_opposite_pos: Vec<SingleLineSpan>,
 }
 
+fn split_comment_words(
+    content: &str,
+    pos: &[SingleLineSpan],
+    opposite_content: &str,
+    prev_opposite_pos: &[SingleLineSpan],
+) -> Vec<MatchedPos> {
+    // TODO: also split on whitespace, so "// (foo)" splits before "(".
+
+    // TODO: don't highlight individual word changes if the lines are
+    // largely novel (e.g. compute levenshtein distance).
+
+    // TODO: merge adjacent single-line comments unless there are
+    // blank lines between them.
+    let word_boundary = Regex::new(r"\b").unwrap();
+
+    let content_parts: Vec<_> = word_boundary.split(content).collect();
+    let other_parts: Vec<_> = word_boundary.split(opposite_content).collect();
+
+    let content_newlines = NewlinePositions::from(content);
+
+    let mut offset = 0;
+
+    let mut res = vec![];
+    for diff_res in diff::slice(&content_parts, &other_parts) {
+        match diff_res {
+            diff::Result::Left(word) => {
+                // This word is novel to this side.
+                res.push(MatchedPos {
+                    kind: MatchKind::ChangedCommentPart,
+                    pos: content_newlines.from_offsets_relative_to(
+                        pos[0],
+                        offset,
+                        offset + word.len(),
+                    ),
+                    prev_opposite_pos: prev_opposite_pos.to_vec(),
+                });
+                offset += word.len();
+            }
+            diff::Result::Both(word, _) => {
+                // This word is present on both sides.
+                res.push(MatchedPos {
+                    kind: MatchKind::UnchangedCommentPart,
+                    pos: content_newlines.from_offsets_relative_to(
+                        pos[0],
+                        offset,
+                        offset + word.len(),
+                    ),
+                    prev_opposite_pos: prev_opposite_pos.to_vec(),
+                });
+                offset += word.len();
+            }
+            diff::Result::Right(_) => {
+                // Only exists on other side, nothing to do on this side.
+            }
+        }
+    }
+
+    res
+}
+
+impl MatchedPos {
+    fn new(
+        ck: ChangeKind,
+        pos: Vec<SingleLineSpan>,
+        prev_opposite_pos: Vec<SingleLineSpan>,
+    ) -> Vec<Self> {
+        let kind = match ck {
+            ReplacedComment(this, opposite) => {
+                let this_content = match this {
+                    List { .. } => unreachable!(),
+                    Atom { content, .. } => content,
+                };
+                let opposite_content = match opposite {
+                    List { .. } => unreachable!(),
+                    Atom { content, .. } => content,
+                };
+
+                return split_comment_words(
+                    this_content,
+                    &pos,
+                    opposite_content,
+                    &prev_opposite_pos,
+                );
+            }
+            Unchanged(_) => MatchKind::Unchanged,
+            Novel => MatchKind::Novel,
+        };
+
+        vec![Self {
+            kind,
+            pos,
+            prev_opposite_pos,
+        }]
+    }
+}
+
 /// Walk `nodes` and return a vec of all the changed positions.
 pub fn change_positions<'a>(
     src: &str,
@@ -438,13 +579,14 @@ fn change_positions_<'a>(
     for node in nodes {
         match node {
             List {
-                change,
+                info,
                 open_position,
                 children,
                 close_position,
                 ..
             } => {
-                let change = change
+                let change = info
+                    .change
                     .get()
                     .unwrap_or_else(|| panic!("Should have changes set in all nodes: {:#?}", node));
 
@@ -460,11 +602,11 @@ fn change_positions_<'a>(
                     }
                 }
 
-                positions.push(MatchedPos {
-                    kind: MatchKind::from_change(change),
-                    pos: open_position.clone(),
-                    prev_opposite_pos: prev_opposite_pos.clone(),
-                });
+                positions.extend(MatchedPos::new(
+                    change,
+                    open_position.clone(),
+                    prev_opposite_pos.clone(),
+                ));
 
                 change_positions_(
                     nl_pos,
@@ -485,16 +627,15 @@ fn change_positions_<'a>(
                         Atom { .. } => unreachable!(),
                     }
                 }
-                positions.push(MatchedPos {
-                    kind: MatchKind::from_change(change),
-                    pos: close_position.clone(),
-                    prev_opposite_pos: prev_opposite_pos.clone(),
-                });
+                positions.extend(MatchedPos::new(
+                    change,
+                    close_position.clone(),
+                    prev_opposite_pos.clone(),
+                ));
             }
-            Atom {
-                change, position, ..
-            } => {
-                let change = change
+            Atom { info, position, .. } => {
+                let change = info
+                    .change
                     .get()
                     .unwrap_or_else(|| panic!("Should have changes set in all nodes: {:#?}", node));
                 if let Unchanged(opposite_node) = change {
@@ -511,11 +652,11 @@ fn change_positions_<'a>(
                         }
                     }
                 }
-                positions.push(MatchedPos {
-                    kind: MatchKind::from_change(change),
-                    pos: position.clone(),
-                    prev_opposite_pos: prev_opposite_pos.clone(),
-                });
+                positions.extend(MatchedPos::new(
+                    change,
+                    position.clone(),
+                    prev_opposite_pos.clone(),
+                ));
             }
         }
     }
@@ -528,7 +669,7 @@ fn zip_pad_shorter<Tx: Copy, Ty: Copy>(lhs: &[Tx], rhs: &[Ty]) -> Vec<(Option<Tx
     loop {
         match (lhs.get(i), rhs.get(i)) {
             (None, None) => break,
-            (x, y) => res.push((x.map(|v| *v), y.map(|v| *v))),
+            (x, y) => res.push((x.copied(), y.copied())),
         }
 
         i += 1;
@@ -573,15 +714,17 @@ pub fn aligned_lines(
     rhs_lines: &[LineNumber],
     lhs_line_matches: &HashMap<LineNumber, LineNumber>,
 ) -> Vec<(Option<LineNumber>, Option<LineNumber>)> {
-    let mut rhs_lines_available: HashSet<_> = rhs_lines.iter().collect();
+    let mut rhs_highest_matched = rhs_lines.first().map(|l| l.number as isize).unwrap_or(0) - 1;
 
-    // For every LHS line, if there is a RHS line that included in
-    // `rhs_lines` and hasn't yet been paired up, add it to matched_lines.
+    // For every LHS line, if there is a RHS line that is included in
+    // `rhs_lines` and hasn't yet been paired up, add it to
+    // matched_lines.
     let mut matched_lines = vec![];
     for lhs_line in lhs_lines {
         if let Some(rhs_line) = lhs_line_matches.get(lhs_line) {
-            if rhs_lines_available.remove(&rhs_line) {
+            if rhs_line.number as isize > rhs_highest_matched {
                 matched_lines.push((lhs_line, rhs_line));
+                rhs_highest_matched = rhs_line.number as isize;
             }
         }
     }
@@ -633,7 +776,7 @@ pub fn matching_lines<'a>(nodes: &[&Syntax<'a>]) -> HashMap<LineNumber, LineNumb
 fn matching_lines_<'a>(node: &Syntax<'a>, matches: &mut HashMap<LineNumber, LineNumber>) {
     match node {
         List {
-            change,
+            info,
             open_position,
             children,
             close_position,
@@ -643,18 +786,14 @@ fn matching_lines_<'a>(node: &Syntax<'a>, matches: &mut HashMap<LineNumber, Line
                 open_position: other_open,
                 close_position: other_close,
                 ..
-            })) = change.get()
+            })) = info.change.get()
             {
                 for (line, other_line) in zip_lines(open_position, other_open) {
-                    if !matches.contains_key(&line) {
-                        matches.insert(line, other_line);
-                    }
+                    matches.entry(line).or_insert(other_line);
                 }
 
                 for (line, other_line) in zip_lines(close_position, other_close) {
-                    if !matches.contains_key(&line) {
-                        matches.insert(line, other_line);
-                    }
+                    matches.entry(line).or_insert(other_line);
                 }
             }
 
@@ -662,18 +801,14 @@ fn matching_lines_<'a>(node: &Syntax<'a>, matches: &mut HashMap<LineNumber, Line
                 matching_lines_(child, matches);
             }
         }
-        Atom {
-            change, position, ..
-        } => {
+        Atom { info, position, .. } => {
             if let Some(Unchanged(Atom {
                 position: other_pos,
                 ..
-            })) = change.get()
+            })) = info.change.get()
             {
                 for (line, other_line) in zip_lines(position, other_pos) {
-                    if !matches.contains_key(&line) {
-                        matches.insert(line, other_line);
-                    }
+                    matches.entry(line).or_insert(other_line);
                 }
             }
         }
@@ -684,7 +819,6 @@ fn matching_lines_<'a>(node: &Syntax<'a>, matches: &mut HashMap<LineNumber, Line
 mod tests {
     use super::*;
     use pretty_assertions::assert_eq;
-    use AtomKind::Other;
 
     #[test]
     fn test_aligned_middle() {
@@ -750,23 +884,74 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_aligned_out_of_order() {
+        let lhs_lines: Vec<LineNumber> = vec![1.into(), 2.into()];
+        let rhs_lines: Vec<LineNumber> = vec![11.into(), 12.into()];
+
+        let mut line_matches: HashMap<LineNumber, LineNumber> = HashMap::new();
+        line_matches.insert(2.into(), 11.into());
+        line_matches.insert(1.into(), 12.into());
+
+        assert_eq!(
+            aligned_lines(&lhs_lines, &rhs_lines, &line_matches),
+            vec![
+                (None, Some(11.into())),
+                (Some(1.into()), Some(12.into())),
+                (Some(2.into()), None)
+            ]
+        );
+    }
+
+    #[test]
+    fn test_aligned_out_of_range() {
+        let lhs_lines: Vec<LineNumber> = vec![1.into(), 2.into()];
+        let rhs_lines: Vec<LineNumber> = vec![11.into(), 12.into()];
+
+        let mut line_matches: HashMap<LineNumber, LineNumber> = HashMap::new();
+        line_matches.insert(1.into(), 10.into());
+
+        assert_eq!(
+            aligned_lines(&lhs_lines, &rhs_lines, &line_matches),
+            vec![
+                (Some(1.into()), Some(11.into())),
+                (Some(2.into()), Some(12.into())),
+            ]
+        );
+    }
+
+    #[test]
+    fn test_aligned_first_line() {
+        let lhs_lines: Vec<LineNumber> = vec![0.into()];
+        let rhs_lines: Vec<LineNumber> = vec![0.into()];
+
+        let mut line_matches: HashMap<LineNumber, LineNumber> = HashMap::new();
+        line_matches.insert(0.into(), 0.into());
+
+        assert_eq!(
+            aligned_lines(&lhs_lines, &rhs_lines, &line_matches),
+            vec![(Some(0.into()), Some(0.into()))]
+        );
+    }
+
     /// Ensure that we assign prev_opposite_pos even if the change is on the first node.
     #[test]
     fn test_prev_opposite_pos_first_node() {
-        // TODO: use Node::new_atom
-        let nodes = &[&Atom {
-            pos_content_hash: 0,
-            next: Cell::new(None),
-            change: Cell::new(Some(Novel)),
-            position: vec![SingleLineSpan {
+        let arena = Arena::new();
+
+        let atom = Syntax::new_atom(
+            &arena,
+            vec![SingleLineSpan {
                 line: 0.into(),
                 start_col: 2,
                 end_col: 3,
             }],
-            content: "foo".into(),
-            kind: Other,
-        }];
-        let positions = change_positions("irrelevant", "also irrelevant", nodes);
+            "foo".into(),
+        );
+        atom.set_change(ChangeKind::Novel);
+        let nodes: Vec<&Syntax> = vec![atom];
+
+        let positions = change_positions("irrelevant", "also irrelevant", &nodes);
         assert_eq!(
             positions[0].prev_opposite_pos,
             vec![SingleLineSpan {
@@ -777,33 +962,62 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_comment_and_atom_differ() {
+        let pos = vec![SingleLineSpan {
+            line: 0.into(),
+            start_col: 2,
+            end_col: 3,
+        }];
+
+        let arena = Arena::new();
+
+        let comment = Syntax::new_comment(&arena, pos.clone(), "foo".into());
+        let atom = Syntax::new_atom(&arena, pos, "foo".into());
+
+        assert_ne!(comment, atom);
+    }
+
     #[test]
     fn test_atom_equality_ignores_change() {
         assert_eq!(
             Atom {
-                pos_content_hash: 0,
-                next: Cell::new(None),
-                change: Cell::new(Some(Novel)),
+                info: SyntaxInfo {
+                    unique_id: Cell::new(0),
+                    pos_content_hash: 0,
+                    next: Cell::new(None),
+                    change: Cell::new(Some(Novel)),
+                    num_ancestors: Cell::new(0),
+                },
                 position: vec![SingleLineSpan {
                     line: 1.into(),
                     start_col: 2,
                     end_col: 3
                 }],
                 content: "foo".into(),
-                kind: Other,
+                is_comment: false,
             },
             Atom {
-                pos_content_hash: 1,
-                next: Cell::new(None),
-                change: Cell::new(None),
+                info: SyntaxInfo {
+                    unique_id: Cell::new(1),
+                    pos_content_hash: 1,
+                    next: Cell::new(None),
+                    change: Cell::new(None),
+                    num_ancestors: Cell::new(1),
+                },
                 position: vec![SingleLineSpan {
                     line: 1.into(),
                     start_col: 2,
                     end_col: 3
                 }],
                 content: "foo".into(),
-                kind: Other,
+                is_comment: false,
             }
         );
     }
+
+    // #[test]
+    // fn test_split_comment_words() {
+
+    // }
 }
